<?php
namespace Drupal\book_pdfs;

use Dompdf\Dompdf;
use Drupal\book\BookExport;
use Drupal\book_pdfs\Event\BookPdfsPreGenerateEvent;
use Drupal\book_pdfs\Event\BookPdfsEvents;
use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\node\Entity\Node;
use Psr\Log\LoggerInterface;

/**
 * BookPdfsGenerator service for book_pdfs using dompdf.
 */
class BookPdfsGenerator {

  const CACHE_DIR = 'public://book_pdfs/';

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The Book export service.
   *
   * @var \Drupal\book\BookExport
   */
  protected $bookExport;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The book_pdf logger channel.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The book_pdf settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  /**
   * The event dispatcher.
   *
   * @var \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher
   */
  protected $eventDispatcher;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The DomPdf library.
   * 
   * @var \Dompdf\Dompdf
   */
  protected $dompdf;

  /**
   * Constructs a BookPdfsGenerator object.
   *
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\book\BookExport $book_export
   *   The Book export service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Psr\Log\LoggerInterface $logger_channel
   *   The logger.channel.book_pdf service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Dompdf\Dompdf
   *   The dompdf service.
   */
  public function __construct(StateInterface $state, BookExport $book_export, RendererInterface $renderer, LoggerInterface $logger_channel, ConfigFactoryInterface $config_factory, ContainerAwareEventDispatcher $event_dispatcher, LanguageManagerInterface $language_manager, Dompdf $dompdf) {
    $this->state = $state;
    $this->bookExport = $book_export;
    $this->renderer = $renderer;
    $this->logger = $logger_channel;
    $this->settings = $config_factory->get('book_pdfs.settings');
    $this->eventDispatcher = $event_dispatcher;
    $this->languageManager = $language_manager;
    $this->dompdf = $dompdf;
  }

  /**
   * Get the Book PDFs cache filename.
   *
   * @param \Drupal\node\Entity\Node $book
   *   The book to get the cache filename for.
   *
   * @return string
   *   The filename.
   */
  protected function getCacheFilename(Node $book) {
    // Different cache per langcode.
    $langcode = $this->languageManager->getCurrentLanguage()->getId();

    if ($book->getEntityType()->isRevisionable()) {
      $vid = $book->getRevisionId();
      return $book->book['bid'] . "-$vid-$langcode" . '.pdf';
    }
    return $book->book['bid'] . "-$langcode" . '.pdf';
  }

  /**
   * Get the Book PDFs contents using dompdf.
   *
   * @param \Drupal\node\Entity\Node $book
   *   The Book node.
   *
   * @return bool|string
   */
  protected function getPdfContents(Node $book) {
    // Check if custom CSS is enabled in the configuration.
    $css_enabled = $this->settings->get('css_enabled');
    $css_file = 'public://book_pdfs/book_pdfs.css';
    $file_exists = file_exists(\Drupal::service('file_system')->realpath($css_file));

    $html = '';
    // If CSS is enabled and the CSS file exists, include it in the HTML.
    if ($css_enabled && $file_exists) {
      $css_content = file_get_contents(\Drupal::service('file_system')->realpath($css_file));
      $html = '<style>' . $css_content . '</style>' . $html;
    }

    $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $event = new BookPdfsPreGenerateEvent($book, $this->dompdf, ['html' => $html], $langcode);
    $this->eventDispatcher->dispatch($event, BookPdfsEvents::PRE);
    $this->dompdf = $event->getPdf();

    $this->dompdf->loadHtml($event->getHtml());

    // Get the paper size from settings.
    $paper_size = $this->settings->get('paper_size') ?? 'letter';

    // Set the paper size and orientation.
    $this->dompdf->setPaper($paper_size, 'portrait');

    $this->dompdf->set_base_path('public://book_pdfs/');
    // Enable remote resources (images, styles).
    $this->dompdf->set_option('isRemoteEnabled', TRUE);
    $this->dompdf->set_option('enable_remote', TRUE);
    // Render the PDF (convert HTML to PDF).
    $this->dompdf->render();

    if ($content = $this->dompdf->output()) {
      // Return the PDF as a string.
      return $content;
    }
    else {
      $this->logger->error('Error generating Book PDF. <br> %error', ['%error' => $pdf->getError()]);
      return FALSE;
    }
  }

  /**
   * Renders parameter as HTML.
   */
  protected function renderHtml(string $html = '') {
    return $this->renderer->render($html);
  }

  /**
   * Renders the book node as HTML.
   */
  protected function renderBookHtml(Node $book, string $html = '') {
    $bookBuild = $this->bookExport->bookExportHtml($book);
    return $this->renderer->render($bookBuild);
  }

  /**
   * Function to fetch remote resource with basic authentication.
   */
  protected function fetchRemoteResource($url) {
    // Basic auth configuration.
    $username = $this->settings->get('basic_user');
    $password = $this->settings->get('basic_pass');
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    if (!empty($username) && !empty($password)) {
      curl_setopt($ch, CURLOPT_USERPWD, "$username:$password");
      curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
    }
    $response = curl_exec($ch);
    curl_close($ch);
  
    return $response;
  }

  /**
   * Get the state key.
   *
   * @param \Drupal\node\Entity\Node $book
   *   The book to get the cache state key for.
   *
   * @return string
   *   The state key.
   */
  public function getStateKey(Node $book) {
    $langcode = $this->languageManager->getCurrentLanguage()->getId();

    if ($book->getEntityType()->isRevisionable()) {
      $vid = $book->getRevisionId();
      return 'book_pdfs_' . $book->book['bid'] . "_$vid" . "_$langcode";
    }
    return 'book_pdfs_' . $book->book['bid'] . "_$langcode";
  }

  /**
   * Get the URI for a Book's cached PDF.
   *
   * @param \Drupal\node\Entity\Node $book
   *   The Book to return the PDF file path for.
   *
   * @return bool|string
   *   The URI if the file can generated\retrieved from cache or
   *   FALSE if something went wrong.
   */
  public function getFileUri(Node $book) {
    if (\Drupal::currentUser()->hasPermission('bypass pdf caching') == FALSE) {
      $bookCached = $this->state->get($this->getStateKey($book), FALSE);
      if ($bookCached && file_exists($bookCached)) {
        return $bookCached;
      }
    }
    $book = \Drupal::service('entity.repository')->getTranslationFromContext($book);
    if ($pdfContents = $this->getPdfContents($book)) {
      if (!is_dir($this::CACHE_DIR) && !mkdir($this::CACHE_DIR)) {
        $this->logger->error('Error creating Book PDF cache directory.');
        return FALSE;
      }
      $fileName = $this::CACHE_DIR . $this->getCacheFilename($book);
      if (file_put_contents($fileName, $pdfContents)) {
        $this->state->set($this->getStateKey($book), $fileName);
        return $fileName;
      }
      else {
        $this->logger->error('Error saving cached book PDF to disk.');
        return FALSE;
      }
    }
    else {
      return FALSE;
    }
  }
}

