<?php

namespace Drupal\book_pdfs\Form;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Book PDFs settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'book_pdfs_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['book_pdfs.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('book_pdfs.settings');

    $form['basic_auth'] = [
      '#type' => 'fieldset',
      '#title' => 'Basic authentication',
      '#description' => 'Configure the Basic auth username and password phpwkhtmltopdf should use fetching remote assets while generating the PDF',
    ];
    $form['basic_auth']['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#default_value' => $config->get('basic_user'),
    ];
    $form['basic_auth']['password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#default_value' => $config->get('basic_pass'),
    ];

    // Add settings for booktree_deep and booktree_trim
    $form['booktree_deep'] = [
      '#type' => 'number',
      '#title' => $this->t('Booktree Depth'),
      '#default_value' => $config->get('booktree_deep') ?? 5,
      '#description' => $this->t('Maximum depth for the book tree.'),
      '#required' => TRUE,
    ];
    $form['booktree_trim'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Booktree Title Trim Length'),
      '#default_value' => $config->get('booktree_trim') ?? 35,
      '#description' => $this->t('Maximum length of the book titles in the tree.'),
      '#required' => TRUE,
    ];

    $form['file_prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('PDF file prefix'),
      '#default_value' => $config->get('file_prefix') ?? 'book-',
      '#description' => 'Optional. In the menu, the heavier items will sink and the lighter items will be positioned nearer the top.',
      '#element_validate' => [[static::class, 'validateFilePrefixElement']],
    ];

    // Add checkbox for TOC mode.
    $form['toc_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Table of Contents'),
      '#default_value' => $config->get('toc_mode') ?? FALSE,
      '#description' => $this->t('Enable or disable the Table of Contents in the generated PDF.'),
    ];

    $form['custom_css'] = array(
      '#type' => 'details',
      '#title' => t('Custom CSS'),
      '#open' => TRUE,
      '#tree' => TRUE,
    );

    // Switch to enable/disable customization.
    $form['custom_css']['css_enabled'] = array(
      '#title' => t('Enable or disable custom CSS:'),
      '#type' => 'checkbox',
      '#default_value' => $config->get('css_enabled') ?? FALSE,
    );
    // Editor box.
    $form['custom_css']['css'] = array(
      '#type' => 'textarea',
      '#prefix' => '<div id="css-editor-field">',
      '#description' => t('Type or paste custom CSS code for this theme.'),
      '#default_value' => $config->get('css') ?? NULL,
      '#attributes' => array('id' => 'css-editor-textarea'),
      '#suffix' => '</div>',
    );
    // Switch to enable/disable the plain text editor.
    $form['custom_css']['plaintext_enabled'] = array(
      '#title' => t('Use plain text editor'),
      '#type' => 'checkbox',
      '#default_value' => $config->get('plaintext_enabled') ?? FALSE,
      '#attributes' => [
        'id' => 'edit-plaintext-enabled',
      ],
    );
    $form['pdf_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('PDF Settings'),
      '#description' => $this->t('Configure PDF generation settings.'),
    ];

    // Add paper size selection.
    $form['pdf_settings']['paper_size'] = [
      '#type' => 'select',
      '#title' => $this->t('Paper Size'),
      '#options' => [
        'letter' => $this->t('Letter'),
        'a4' => $this->t('A4'),
        'legal' => $this->t('Legal'),
        // Add more sizes as needed.
      ],
      '#default_value' => $config->get('paper_size') ?? 'letter',
      '#description' => $this->t('Select the default paper size for the PDF.'),
    ];

    $form['header_footer'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Header and Footer'),
      '#description' => $this->t('Configure custom headers and footers for the PDF.'),
    ];
    $form['header_footer']['header'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Header Content'),
      '#description' => $this->t('Define custom HTML for the header. Leave empty for no header.'),
      '#default_value' => $config->get('header') ?? '',
    ];
    $form['header_footer']['footer'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Footer Content'),
      '#description' => $this->t('Define custom HTML for the footer. Use {{page}} for current page and {{total}} for total pages. Leave empty for no footer.'),
      '#default_value' => $config->get('footer') ?? '',
    ];
    $form['splash_page'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Splash Page Content'),
      '#description' => $this->t('HTML content to be used as a splash page at the beginning of the PDF. Leave empty for no splash page.'),
      '#default_value' => $config->get('splash_page') ?? '',
    ];
    
    // Add the "Delete PDF Cache" button.
    $form['delete_pdf_cache'] = [
      '#type' => 'submit',
      '#value' => $this->t('Delete PDF Cache'),
      '#submit' => ['::deletePdfCache'],
      '#description' => $this->t('This will delete all cached PDF files in the public://book_pdfs directory.'),
    ];

    // Attach CSS and Javascript libraries.
    $form['#attached']['library'][] = 'book_pdfs/css_editor';
    $form['#attached']['library'][] = 'book_pdfs/codemirror';
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save file.
    $path = 'public://book_pdfs';
    $file = $path . DIRECTORY_SEPARATOR . 'book_pdfs.css';
    if (\Drupal::service('file_system')->prepareDirectory($path, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
      \Drupal::service('file_system')->saveData($form_state->getValue(['custom_css', 'css']), $file, FileSystemInterface::EXISTS_REPLACE);
      \Drupal::service('file_system')->chmod($file);
    }
    $this->config('book_pdfs.settings')
      ->set('css_enabled', $form_state->getValue(['custom_css', 'css_enabled']))
      ->set('css', $form_state->getValue(['custom_css', 'css']))
      ->set('splash_page', $form_state->getValue('splash_page'))
      ->set('header', $form_state->getValue('header'))
      ->set('footer', $form_state->getValue('footer'))
      ->set('paper_size', $form_state->getValue('paper_size'))
      ->set('plaintext_enabled', $form_state->getValue(['custom_css', 'plaintext_enabled']))
      ->set('basic_user', $form_state->getValue('username'))
      ->set('basic_pass', $form_state->getValue('password'))
      ->set('file_prefix', $form_state->getValue('file_prefix'))
      ->set('booktree_deep', $form_state->getValue('booktree_deep'))
      ->set('booktree_trim', $form_state->getValue('booktree_trim'))
      ->set('toc_mode', $form_state->getValue('toc_mode'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Custom submit handler to delete cached PDF files.
   */ 
  public function deletePdfCache(array &$form, FormStateInterface $form_state) {
    $path = 'public://book_pdfs';
    $file_system = \Drupal::service('file_system'); 
    
    if ($file_system->prepareDirectory($path, FileSystemInterface::CREATE_DIRECTORY)) {
      // Load all .pdf files in the directory.
      $pdf_files = $file_system->scanDirectory($path, '/\.pdf$/');
      
      if (!empty($pdf_files)) {
        foreach ($pdf_files as $file_info) {
          $file_system->delete($file_info->uri);
        }
        \Drupal::messenger()->addMessage($this->t('All cached PDF files have been deleted.'));
      }
      else {
        \Drupal::messenger()->addMessage($this->t('No cached PDF files found.'), 'warning');
      }
    }
    else {
      \Drupal::messenger()->addError($this->t('Failed to prepare the directory.'));
    }
  }

  /**
   * Form element validation handler for the 'file_prefix' element.
   *
   * Only allow letters, numbers, underscores and dashes.
   */
  public static function validateFilePrefixElement($element, FormStateInterface $form_state, $form) {
    $file_prefix = trim($element['#value']);
    $file_prefix = str_replace(' ', '-', $file_prefix);
    $form_state->setValueForElement($element, $file_prefix);

    if (!preg_match('/^[a-zA-Z0-9-_]+$/', $file_prefix)) {
      $form_state->setError($element, t('Allowed characters are a-z, 0-9, \'-\', and \'_\'.'));
      return;
    }
  }
}

